"""Cookbook API."""

import argparse
import logging
from abc import ABCMeta, abstractmethod
from logging import Logger

from clippy.api import Clippy


class ArgparseFormatter(argparse.ArgumentDefaultsHelpFormatter):
    """Custom argparse formatter class for cookbooks."""


class Cookbook(metaclass=ABCMeta):
    args: argparse.Namespace

    clippy: Clippy

    unlisted: bool = False

    def __init__(self, name: str, clippy: Clippy):
        self.name = name
        self.clippy = clippy

    @classmethod
    @property
    def title(cls) -> str:
        if cls.__doc__ is None:
            return "-"

        return cls.__doc__.splitlines()[0]

    def argument_parser(self) -> argparse.ArgumentParser:
        return argparse.ArgumentParser(
            prog=f"clippy [GLOBAL_ARGS] {self.name}", description=self.__doc__, formatter_class=ArgparseFormatter
        )

    @property
    def logger(self) -> Logger:
        logger = logging.getLogger(self.name)
        logger.setLevel(logging.getLogger("clippy").getEffectiveLevel())
        return logger

    @abstractmethod
    def execute(self):
        pass
