import logging
import os
import sys


def enable_color():
    if os.getenv("NO_COLOR"):
        return False
    return sys.stdout.isatty()


class Color:
    """ANSI color code"""

    ENABLED = enable_color()

    def __init__(self, code):
        self.code = code

    def __call__(self, text):
        if not Color.ENABLED:
            return text

        return "\x1b[{code}m{text}\x1b[39m".format(code=self.code, text=text)


class Colors:
    RED = Color(31)
    GREEN = Color(32)
    YELLOW = Color(33)
    BLUE = Color(34)
    MAGENTA = Color(35)
    CYAN = Color(36)
    GRAY = Color(37)


class ClippyLogFormatter(logging.Formatter):
    def __init__(self):
        super().__init__(
            f"{Colors.GRAY('%(asctime)s')} %(levelname)s %(message)s",
            datefmt="%H:%M:%S",
        )

    def formatMessage(self, record: logging.LogRecord) -> str:
        record.levelname = self.color_levelname(record.levelname, record.levelno)
        return super().formatMessage(record)

    def color_levelname(self, name: str, level: int) -> str:
        if level == logging.INFO:
            return Colors.YELLOW(" ==> ")
        elif level == logging.DEBUG:
            return Colors.BLUE("DEBUG")
        elif level == logging.WARNING:
            return Colors.YELLOW(" WARN")
        elif level == logging.ERROR:
            return Colors.RED("ERROR")

        return name
